@@ -15,49 +15,49 @@ class CLASS_NAME {
 
   #ifdef CheckDimensions
 
-    TYPE value(size_t i) {
-      if (i<dims[1]) {
-        return data_[i];
-      } else {
-        throw (Exception(
-            string("Index out of range in variable" + name_).c_str()));
+    TYPE value(const size_t i) {
+      // fail fast
+      if(i >= dims[1]) {
+        stop("Index out of range in variable %s", name_);
       }
+
+      return data_[i];
     }
 
-    TYPE value(size_t i, size_t j) {
-      if (dims.size() == 2) {
-           if ( (i < dims[0]) && (j < dims[1]) ) {
-             return data_[j * dims[0] + i];
-           } else {
-             throw (Exception(
-                 string("Index out of range in variable" + name_)));
-           }
-      } else {
-        throw (Exception(
-            string("incorrect number of dimensions accessing variable" + name_)));
+    TYPE value(const size_t i, const size_t j) {
+      // fail fast
+      if(dims.size() != 2) {
+        stop("incorrect number of dimensions accessing variable %s", name_);
       }
+
+      // fail fast
+      if ( (i >= dims[0]) || (j >= dims[1]) ) {
+        stop("Index out of range in variable %s", name_);
+      }
+
+      return data_[j * dims[0] + i];
     }
 
-    TYPE linValue(size_t i) {
+    TYPE linValue(const size_t i) {
       size_t max = 1;
 
       for (size_t di = 0; di < dims.size(); di++) {
         max *= dims[di];
       }
 
-      if (i < max) {
-        return data_[i];
-      } else {
-        throw (Exception(
-            string("Linear index out of range in variable" + name_)));
+      // fail fast
+      if (i >= max) {
+        stop("Linear index out of range in variable %s", name_);
       }
+
+      return data_[i];
     }
 
     void setValue(size_t i, TYPE r) {
       if (i < dims[0]) {
         data_[i] = r;
       } else {
-        throw (Exception(string("Index out of range in variable" + name_)));
+        stop("Index out of range in variable %s", name_);
       }
     }
 
@@ -66,11 +66,10 @@ class CLASS_NAME {
         if ( (i < dims[0]) && (j < dims[1]) ) {
           data_[j * dims[0] + i] = r;
         } else {
-          throw (Exception(string("Index out of range in variable" + name_)));
+          stop("Index out of range in variable %s", name_);
         }
       } else {
-        throw (Exception(
-            string("incorrect number of dimensions accessing variable" + name_)));
+        stop("incorrect number of dimensions accessing variable %s", name_);
       }
     }
 
@@ -79,11 +78,10 @@ class CLASS_NAME {
         if ((k < dims[2]) && (j < dims[1]) && (i < dims[0])) {
           data_[(k * dims[1] + j) * dims[0] + i] = r;
         } else {
-          throw (Exception(string("Index out of range in variable" + name_)));
+          stop("Index out of range in variable", name_);
         }
       } else {
-        throw (Exception(
-            string("Incorrect number of dimensions accessing variable" + name_)));
+        stop("Incorrect number of dimensions accessing variable %s", name_);
       }
     }
 
@@ -96,8 +94,7 @@ class CLASS_NAME {
       if (i < max) {
         data_[i] = r;
       } else {
-        throw (Exception(
-            string("Linear index out of range in variable" + name_)));
+        stop("Linear index out of range in variable %s", name_);
       }
     }
 
@@ -192,11 +189,8 @@ class CLASS_NAME {
 
 void CLASS_NAME::setDim(size_t length) {
   if (length > size_) {
-    throw (Exception(
-        "attempt to set linear dimension " +
-        NumberToString(length) + " higher than size " +
-        NumberToString(size()) + " in variable " + name()
-    ));
+    stop("attempt to set linear dimension %d higher than size %d in variable %s",
+         length, size(), name());
   } else {
     dims.clear();
     dims.push_back(length);
@@ -205,11 +199,8 @@ void CLASS_NAME::setDim(size_t length) {
 
 void CLASS_NAME::setDim(size_t nrow, size_t ncol) {
   if (nrow*ncol > size()) {
-    throw (Exception(
-        "attempt to set matrix dimensions " + NumberToString(nrow) + ", " +
-        NumberToString(ncol) + " higher than size " +
-        NumberToString(size()) + " in variable " + name()
-    ));
+    stop("attempt to set matrix dimensions %d, %d higher than size %d in variable %s",
+         nrow, ncol, size(), name());
   } else {
     dims.clear();
     dims.push_back(nrow);
@@ -224,8 +215,7 @@ void CLASS_NAME::setDim(vector <size_t> dims, size_t start) {
   }
 
   if (len > size()) {
-    throw(Exception(
-        string("setDim: not enough space to accomodate given dimensions.")));
+    stop("setDim: not enough space to accomodate given dimensions.");
   }
 
   this->dims.clear();
